function eta, eta0, vf, a, vb, gamma, chi

; Computes Absorption coefficients from Jefferies, Lites, and Skumanich

;--------------------------------------------------------------
;   Weak field approximation to the eta's and rho's
;
;   etaI = eta0*(H+(1.0+cos(gamma)^2)*Hpp*(vb/2.)^2)
;   etaQ = -eta0*((vb*sin(gamma)/2.)^2*Hpp*cos(2.*chi))
;   etaU = -eta0*((vb*sin(gamma)/2.)^2*Hpp*sin(2.*chi))
;   etaV = eta0*vb*cos(gamma)*Hp
;
;   rhoQ = -eta0*((vb*sin(gamma)/2.)^2*Fpp*cos(2.*chi))
;   rhoU = -eta0*((vb*sin(gamma)/2.)^2*Fpp*sin(2.*chi))
;   rhoV = eta0*vb*cos(gamma)*Fp
;
;--------------------------------------------------------------

; Following is the "exact" formulation for the eta's and rho's
   
   voigt, a, vf, h, f
   voigt, a, vf+vb, hr, fr
   voigt, a, vf-vb, hl, fl

   kappa_p = eta0 * h
   kappa_r = eta0 * hr
   kappa_l = eta0 * hl
   kappa_avg = (kappa_r+kappa_l)/2.0

   kappa_pp = eta0 * f
   kappa_pr = eta0 * fr
   kappa_pl = eta0 * fl
   kappa_pavg = (kappa_pr+kappa_pl)/2.0

   etaI = 0.5*(kappa_avg*(1.0+(cos(gamma))^2) + kappa_p*(sin(gamma))^2)
   etaQ = 0.5*(kappa_p - kappa_avg)*(sin(gamma))^2*cos(2*chi)
   etaU = 0.5*(kappa_p - kappa_avg)*(sin(gamma))^2*sin(2*chi)
   etaV = 0.5*(kappa_r - kappa_l)*cos(gamma)

   rhoQ = 0.5*(kappa_pp - kappa_pavg)*(sin(gamma))^2*cos(2*chi)
   rhoU = 0.5*(kappa_pp - kappa_pavg)*(sin(gamma))^2*sin(2*chi)
   rhoV = 0.5*(kappa_pr - kappa_pl)*cos(gamma)

   etaIp12 = (1.0 + etaI)^2
   Delta = etaIp12 * ( etaIp12 - etaQ^2 - etaU^2 - etaV^2 + rhoQ^2 + rhoU^2 + $
                       rhoV^2) - ( etaQ*rhoQ + etaU*rhoU + etaV*rhoV)^2

   return,add2str(dummy,['etaI','etaQ','etaU','etaV','rhoQ','rhoU','rhoV', $
                         'Delta','eta0'], $
                        etaI,etaQ,etaU,etaV,rhoQ,rhoU,rhoV,Delta,eta0,/quiet)

   ;command = strcompress('etaval = {eta'+string(unique) + $
   ;          ',etaI:etaI,etaQ:etaQ,etaU:etaU,etaV:etaV'  + $
   ;          ',rhoQ:rhoQ,rhoU:rhoU,rhoV:rhoV,Delta:Delta,eta0:eta0}',/remove_all)

   ;check = execute(command)

   ;if check then $
   ;  return, etaval $
   ;else begin
   ;  print,'WARNING: eta: could not create structure'
   ;  return, -1
   ;endelse

end
