;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;  a bunch of auxiliary routines used by lowandlou.pro:
;    getf,getdfdy,getdfdz,getdfdla,
;    getg,getdgdy,getdgdz,getdgdla,
;    geth,getdhdy,getdhdz,getdhdla,
;    getb1,getdb1dy,getdb1dz,getdb1dla,
;    getb2,getdb2dy,getdb2dz,getdb2dla,
;    getrhs,getlhs
;
;  M.DeRosa -  2 Nov 2004 - created
;             10 Nov 2004 - fixed formulae for when n>1 (but still odd)
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getf,x,y,z,la,_extra=params  ;  returns f(x,y,z;la)

return,z
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdfdy,x,y,z,la,_extra=params  ;  returns partial df/dy

return,make_array(value=0.,dim=n_elements(x))
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdfdz,x,y,z,la,_extra=params  ;  returns partial df/dz

return,make_array(value=1.,dim=n_elements(x))
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdfdla,x,y,z,la,_extra=params  ;  returns partial df/dlambda

return,make_array(value=0.,dim=n_elements(x))
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getg,x,y,z,la,_extra=params  ;  returns g(x,y,z;la)

n=double(params.n(0))
return,-(n*(n+1)*y+la*(1+1/n)*sign_mld(y)*abs(y)^(1+2/n))/(1-x*x)
;return,-(n*(n+1)*y+la*(1+1/n)*y^(1+2/n))/(1-x*x)
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdgdy,x,y,z,la,_extra=params  ;  returns partial dg/dy

n=double(params.n(0))
return,-(n*(n+1)+la*(1+1/n)*(1+2/n)*abs(y)^(2/n))/(1-x*x)
;return,-(n*(n+1)+la*(1+1/n)*(1+2/n)*y^(2/n))/(1-x*x)
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdgdz,x,y,z,la,_extra=params  ;  returns partial dg/dz

return,make_array(value=0.,dim=n_elements(x))
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdgdla,x,y,z,la,_extra=params  ;  returns partial dg/dlambda

n=double(params.n(0))
return,-(1+1/n)*sign_mld(y)*abs(y)^(1+2/n)/(1-x*x)
;return,-(1+1/n)*y^(1+2/n)/(1-x*x)
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getb1,x,y,z,la,_extra=params  ;  returns b1(x,y,z;la)

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdb1dy,x,y,z,la,_extra=params  ;  returns partial db1/dy

return,1.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdb1dz,x,y,z,la,_extra=params  ;  returns partial db1/dz

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdb1dla,x,y,z,la,_extra=params  ;  returns partial db1/dlambda

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getb2,x,y,z,la,_extra=params  ;  returns b2(x,y,z;la)

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdb2dy,x,y,z,la,_extra=params  ;  returns partial db2/dy

return,1.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdb2dz,x,y,z,la,_extra=params  ;  returns partial db2/dz

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdb2dla,x,y,z,la,_extra=params  ;  returns partial db2/dlambda

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function geth,x,y,z,la,_extra=params  ;  returns h(x,y,z;la)

return,-10.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdhdy,x,y,z,la,_extra=params  ;  returns partial dh/dy

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdhdz,x,y,z,la,_extra=params  ;  returns partial dh/dz

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getdhdla,x,y,z,la,_extra=params  ;  returns partial dh/dlambda

return,0.
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getrhs,x,y,z,la,_extra=params  ;  returns RHS of matrix equation

;  preliminaries
nx=n_elements(x)
rhs=dblarr(2*nx+1)
halfdx=0.5*(shift(x,-1)-x)

;  fill in pieces depending on y and function f
oddix=2*indgen(nx)+2
f=getf(x,y,z,la,_extra=params)
rhs(oddix)=shift(y,-1)-y-halfdx*(f+shift(f,-1))

;  fill in pieces depending on z and function g
evenix=2*indgen(nx)+3
g=getg(x,y,z,la,_extra=params)
rhs(evenix)=shift(z,-1)-z-halfdx*(g+shift(g,-1))

;  take care of boundary conditions
;rhs(0)=-y(2)+4.*y(1)-3.*y(0)+$
;  2*(x(1)-x(0))*geth(x(0),y(0),z(0),la,_extra=params)
rhs(0)=y(1)-y(0)+(x(1)-x(0))*geth(x(0),y(0),z(0),la,_extra=params)
rhs(1)=getb1(x(0),y(0),z(0),la,_extra=params)-y(0)
rhs(2*nx)=getb2(x(nx-1),y(nx-1),z(nx-1),la,_extra=params)-y(nx-1)

return,rhs
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function getlhs,x,y,z,la,_extra=params  ;  returns LHS of matrix equation

;  preliminaries
nx=n_elements(x)
lhs=dblarr(2*nx+1,2*nx+1)
halfdx=0.5*(shift(x,-1)-x)

;  fill in pieces depending on y- and z-derivatives of f
ixf=(2*(2*nx+1)+2)*lindgen(nx) + (2*nx+1)*2
ixf(nx-1)=0  ;  avoids out of bounds error, later gets overwritten by BC
fbyy=getdfdy(x,y,z,la,_extra=params)
fbyz=getdfdz(x,y,z,la,_extra=params)
lhs(ixf)=halfdx*shift(fbyy,-1) + 1.
lhs(ixf+1)=halfdx*shift(fbyz,-1)
lhs(ixf+2)=halfdx*fbyy - 1.
lhs(ixf+3)=halfdx*fbyz

;  fill in pieces depending on y- and z-derivatives of g
ixg=(2*(2*nx+1)+2)*lindgen(nx) + (2*nx+1)*3
ixg(nx-1)=0  ;  avoids out of bounds error, later gets overwritten by BC
gbyy=getdgdy(x,y,z,la,_extra=params)
gbyz=getdgdz(x,y,z,la,_extra=params)
lhs(ixg)=halfdx*gbyy
lhs(ixg+1)=halfdx*gbyz + 1.
lhs(ixg+2)=halfdx*shift(gbyy,-1)
lhs(ixg+3)=halfdx*shift(gbyz,-1) - 1.

;  fill in pieces depending on lambda-derivatives of f
ixfla=2*(2*nx+1)*lindgen(nx) + (2*nx+1)*2 + 2*nx
ixfla(nx-1)=0
fbyla=getdfdla(x,y,z,la,_extra=params)
lhs(ixfla)=halfdx*(fbyla+shift(fbyla,-1))

;  fill in pieces depending on lambda-derivatives of g
ixgla=2*(2*nx+1)*lindgen(nx) + (2*nx+1)*3 + 2*nx
ixgla(nx-1)=0
gbyla=getdgdla(x,y,z,la,_extra=params)
lhs(ixgla)=halfdx*(gbyla+shift(gbyla,-1))

;  now deal with boundary conditions
dx=(shift(x,-1)-x)
;lhs(0:5)=[3.-2.*dx(0)*getdhdy(x(0),y(0),z(0),la,_extra=params),$
;  -2*dx(0)*getdhdz(x(0),y(0),z(0),la,_extra=params),-4.,0.,1.,0.]
lhs(0:3)=[1.-dx(0)*getdhdy(x(0),y(0),z(0),la,_extra=params),$
  -dx(0)*getdhdz(x(0),y(0),z(0),la,_extra=params),-1.,0.]  ;  (first order)
;lhs(2*nx)=-2*dx(0)*getdhdla(x(0),y(0),z(0),la,_extra=params)
lhs(2*nx)=-dx(0)*getdhdla(x(0),y(0),z(0),la,_extra=params)  ;  (first order)
lhs(2*nx+1:2*nx+4)=$
  [getdb1dy(x(0),y(0),z(0),la,_extra=params),$
  getdb1dz(x(0),y(0),z(0),la,_extra=params),0.,0.]
lhs(2*nx+1+2*nx)=getdb1dla(x(0),y(0),z(0),la,_extra=params)
lhs((2*nx+1)^2-5:(2*nx+1)^2-1)=$
  [0.,0.,getdb2dy(x(nx-1),y(nx-1),z(nx-1),la,_extra=params),$
  getdb2dz(x(nx-1),y(nx-1),z(nx-1),la,_extra=params),$
  getdb2dla(x(nx-1),y(nx-1),z(nx-1),la,_extra=params)]

return,lhs
end




;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;  now a bunch of routines used by the driver script:
;    gaussquad_legendre, linrange, get_interpolation_index
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;
;  gaussquad_legendre.pro - This procedure computes the Legendre collocation 
;                           points and integration weights over x in (-1,1)
;
;  usage:  gaussquad_legendre,n,x,w
;          where n = number of gridpoints
;                x = n-element array of collocation points
;                w = n-element array of integration weights
;
;  notes:  -based on Numerical Recipes routine gauleg, section 4.5, p.145
;          -maximum number of iterations is about n for eps=1e-6
;          -probably need higher precision variables for n>500 or so
;          -total(w) should equal 2
;
;  M.DeRosa - 2 Oct 2001 - created
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

pro gaussquad_legendre,n,x,w

;  usage message
if n_params() eq 0 then begin
  print,'  gaussquad_legendre,n,x,w'
  return
endif

;  make sure n is legit
order=long(n(0))
if order lt 2 then begin
  print,'  gaussquad_legendre: n must be at least 2'
  return
endif

;  setup
x=dblarr(order,/noz)
pprime=dblarr(order,/noz)

;  set tolerance
eps=1d-6  ;  probably adequate for real*8 double precision

;  loop through points
for i=1l,(order+1)/2 do begin  ;  symmetric domain, only do half of the points

  ;  starting guess for ith root
  guess=cos(!dpi*(i-0.25d)/(order+0.5d))

  ;  iterate until zero is found
  repeat begin

    ;  starting values for P_(n-1) and P_n, evaluated at guess point
    pnm1=1d0
    pn  =guess

    ;  find P_n evaluated at guess point, Afrken (12.17a) after n+1 replaces n
    for j=2l,order do begin
      pnm2=pnm1
      pnm1=pn
      pn=(guess*(2*j-1)*pnm1-(j-1)*pnm2)/j
    endfor

    ;  compute d P_n / dx evaluated at guess point, basically Arfken (12.26)
    dpndx=order*(guess*pn-pnm1)/(guess*guess-1)

    ;  use Newton's method to improve guess
    oldguess=guess
    guess=oldguess-pn/dpndx

  ;  check tolerance, repeat if answer isn't good enough
  endrep until abs(guess-oldguess) le eps

  ;  fill x and pprime arrays
  x(i-1)=-guess
  x(order-i)=guess
  pprime(i-1)=dpndx  ;  may be off by a minus sign, but it gets squared below
  pprime(order-i)=dpndx  ;  ditto above

endfor

;  calculate weights
w=2/((1-x*x)*pprime*pprime)

end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;
;  linrange.pro - creates a double precision array of n numbers starting
;                 with min and ending with max, in a linear fashion.
;
;  usage:  result=linrange(n,max,min)
;          where result = double-precision array with n elements
;                     n = number of points in the array
;                   max = highest number
;                   min = lowest number
;
;  notes:  returns -1 if error detected
;
;  M.DeRosa - 12 Apr 1995 - created
;             30 Nov 1998 - added usage message
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function linrange,n,min,max

;  usage message
if n_params() lt 3 then begin
  print,'  result=linrange(n,min,max)'
  return,-1
endif

np=long(n)
mn=double(min)
mx=double(max)

vec=dindgen(np)
vec=vec*(mx-mn)/(np-1)
vec=vec+mn

return,vec
end

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;
;  get_interpolation_index - This function returns the fractional coordinate 
;                            index of an input array
;
;  usage:  result=get_interpolation_index(array,value)
;            where array = input array
;                  value = values for which index is desired
;
;  notes:  -useful for determining the index for interpolation,  
;           get_interpolation_index(linrange(11,0,2),.68) returns 3.4, since 
;           the value .68 would lie at (fractional) index coordinate 3.4
;          -if out of bounds, then function returns either 0 or N-1, where N
;           is the number of elements in array
;          -assumes that array is monotonically increasing
;          -linear interpolation performed, watch out if array is not equally 
;           spaced
;
;  M.DeRosa - 12 Oct 2001 - created
;             12 Aug 2002 - value argument now takes arrays as well as scalars
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

function get_interpolation_index,array,value

npt=n_elements(value)
out=dblarr(npt,/noz)
for i=0l,npt-1 do begin
  if value(i) le array(0) then begin
    out(i)=0d
  endif else begin
    nearix=where(array gt value(i),nwh)
    if nwh gt 0 then begin
      nix=nearix(0)-1
      extract=array(nix:nix+1)
      out(i)=nix+(value(i)-extract(0))/double(extract(1)-extract(0))
    endif else out(i)=double(n_elements(array)-1)
  endelse
endfor

return,out

end
