;+

function bff_height,ff,h,image=image

;NAME:
;     BFF_HEIGHT
;PURPOSE:
;     Extracts a magnetogram at a specified height index from a force-free 
;     field structure computed with bff.pro
;CATEGORY:
;CALLING SEQUENCE:
;     b = bff_height(ff,h)
;INPUTS:
;     ff = force free field structure from bff.pro
;     h  = height index
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;     /image = return the magnetogram in the image coordinates
;OUTPUTS:
;     b = magnetic field structure which can be plotted with vmgram.pro
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     /image assumes that binup was not set in the call to bff.pro
;PROCEDURE:
;MODIFICATION HISTORY:
;     T. Metcalf 1994-07-01
;-

   message,/info, $
      strcompress('Height: '+string(h*min(ff.pix_size)*ff.zscale*721.9)+' km')

   bx = ff.bxp(*,*,h)
   by = ff.byp(*,*,h)
   bz = ff.bzp(*,*,h)
   point = ff.point

   if keyword_set(image) then begin
      nx = n_elements(ff.bz_obs(*,0))
      ny = n_elements(ff.bz_obs(0,*))
      x = lindgen(nx,ny) MOD nx
      y = long(lindgen(nx,ny) / nx)
      polywarp,x,y,ff.b_pxd-min(ff.b_pxd),ff.b_pyd-min(ff.b_pyd),3,p,q
      polywarp,ff.b_pxd-min(ff.b_pxd),ff.b_pyd-min(ff.b_pyd),x,y,3,pinv,qinv
      bx = poly_2d(bx,pinv,qinv,1,nx,ny,missing=0.)
      by = poly_2d(by,pinv,qinv,1,nx,ny,missing=0.)
      bz = poly_2d(bz,pinv,qinv,1,nx,ny,missing=0.)
      point.p = point.p_true
      point = add2str(point,delete='P_TRUE')
   endif

   return,add2str(dummy,['BX','BY','BZ','POINT'],bx,by,bz,point)

end
