
;+
;NAME:
;     bplot
;PURPOSE:
;     Plots lines of force for a 3D magnetic field from a list of vertices
;CATEGORY:
;CALLING SEQUENCE:
;     bplot,vertices
;INPUTS:
;     vertices = the structure of vertices from the blines output
;     position = plot window specs for contour, oplot etc., [x0,y0,x1,y1,z0,z1]
;     zscale = overrides the zscale in the vertices sturcture
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;     bthick = thickness of the field lines
;     /nofilldot = Don't fill the dot which marks the footpoint (causes 
;                 z-buffer to crash).
;     /quiet = suppress informational messages
;     spot_color = color index for the footpoint spot
;     /nospot = do not plot footpoints
;OUTPUTS:
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;MODIFICATION HISTORY:
;
;       Feb 7, 1992  T. Metcalf
;-

; Plot line of force

pro bplot, vertices, position=position, zscale=zscale, quiet=quiet, $
           bthick=inbthick,nofilldot=nofilldot,spot_color=spot_color, $
           nospot=nospot

   if n_elements(inbthick) NE 1 then bthick=1 else bthick=inbthick
   if n_elements(zscale) LE 0 then zscale = vertices.zscale
   if NOT keyword_set(quiet) then print,'INFO: bplot: zscale = ',zscale
   if n_elements(spot_color) LE 0 then spot_color = 1

   if keyword_set(nofilldot) then $
      usersym,[0.,0.707107,1.,0.707107,0.,-0.707107,-1.,-0.707107], $
              [1.,0.707107,0.,-0.707107,-1.,-0.707107,0.,0.707107] $
   else $
      usersym,[0.,0.707107,1.,0.707107,0.,-0.707107,-1.,-0.707107], $
              [1.,0.707107,0.,-0.707107,-1.,-0.707107,0.,0.707107],/fill

   ; Set up cube for 3D plotting
   
   !X.S = [position(0),(position(2)-position(0))/(vertices.nx-1.-(0.))]
   !Y.S = [position(1),(position(3)-position(1))/(vertices.ny-1.-(0.))]
   ;!Z.S = [position(4),(position(5)-position(4))/(zscale*(vertices.nz-1.)-(0.))]
   ; Set z-scale to same as x,y scales
   ;nzeff = ((1./!X.S(1)) > (1./!Y.S(1)))*(position(5)-position(4))
   !Z.S = [position(4),!X.S(1)<!Y.S(1)]

   lstart = where(vertices.vertices(0,*) EQ -1000,nlines)
   lend = where(vertices.vertices(0,*) EQ -2000,nelines)

   if nlines NE nelines then $
      message,'Incomplete vertices structure: # line starts NE # line ends'

   for line = 0,nlines-1 do begin   ; Plot the lines

     ls1 = lstart(line)+1
     le1 = lend(line)-1

     if le1 GT ls1 then begin

        Bcolor = vertices.vertices(2,lstart(line))
        spot = fix(vertices.vertices(1,lend(line)))

        plots,reform(vertices.vertices(0,ls1:le1)), $
              reform(vertices.vertices(1,ls1:le1)), $
              reform(vertices.vertices(2,ls1:le1)), $
              /T3D,/DATA,color=Bcolor,/noclip,thick=bthick
        if NOT keyword_set(nospot) then begin
           oplot,[vertices.vertices(0,ls1)], $
              [vertices.vertices(1,ls1)], $
              zvalue=0.0,psym=8,/noclip,/t3d, $
              symsize=1.,color=spot_color
        endif
        if spot AND NOT keyword_set(nospot) then begin
           oplot,[vertices.vertices(0,le1)], $
                 [vertices.vertices(1,le1)], $
                 zvalue=0.0,psym=8,/noclip,/t3d, $
                 symsize=1.,color=spot_color
        endif

     endif

   endfor

end
