
; Generate a dipole magnetic field structure

function UofR,x,y,z,alpha,depth
   scale = 1000.*(depth^3)/2
   r = sqrt(x^2+y^2+z^2)
   u = scale*cos(alpha*r)/r
   up = -u/r - scale*alpha*sin(alpha*r)/r
   return,[r,u,up]
end

function dipole,nx,ny,nz,alpha,zscale,depth=depth,bin=bin,one=one

;+
;   d = dipole(nx,ny,nz,alpha,zscale,depth=depth,bin=bin,/one)
;   /one = single dipole (default is two dipoles)
;-


   if n_elements(zscale) LE 0 then zscale = 1
   if n_elements(bin) LE 0 then bin = 1
   if n_elements(depth) LE 0 then depth = 5.

   print,'bin = ',bin

   nx = nx*bin
   ny = ny*bin

   bx = fltarr(nx,ny,nz)
   by = fltarr(nx,ny,nz)
   bz = fltarr(nx,ny,nz)

   for i=0,nx-1 do begin
      for j=0,ny-1 do begin
         for k=0,nz-1 do begin
            if NOT keyword_set(one) then begin
               x = (i-(nx/2)+10.)/bin
               y = (j-ny/2.)/bin
               z = (k+depth)*zscale
            endif $
            else begin
               x = (i-(nx/2))/bin
               y = (j-ny/2.)/bin
               z = (k+depth)*zscale
            endelse
            u = UofR(x,y,z,alpha,depth)
            r = u(0)
            up = u(2)
            u = u(1)
            bx(i,j,k)= (-(3.*up/r + (alpha^2)*u)*x*z/(r^2) + alpha*up*y/r)
            by(i,j,k)= (-(3.*up/r + (alpha^2)*u)*y*z/(r^2) - alpha*up*x/r)
            bz(i,j,k)= ((1.-3.*(z/r)^2)*up/r + (alpha^2)*u*(1.-(z/r)^2))
            if NOT keyword_set(one) then begin
               x = (i-(nx/2)-10.)/bin
               y = (j-ny/2.)/bin
               z = (k+depth)*zscale
               u = UofR(x,y,z,alpha,depth)
               r = u(0)
               up = u(2)
               u = u(1)
               bx(i,j,k)= bx(i,j,k) - $
                          (-(3.*up/r + (alpha^2)*u)*x*z/(r^2) + alpha*up*y/r)
               by(i,j,k)= by(i,j,k) - $
                          (-(3.*up/r + (alpha^2)*u)*y*z/(r^2) - alpha*up*x/r)
               bz(i,j,k)= bz(i,j,k) - $
                          ((1.-3.*(z/r)^2)*up/r + (alpha^2)*u*(1.-(z/r)^2))
            endif
         endfor
      endfor
   endfor

   bpxd = lindgen(nx,ny) MOD nx
   bpyd = long(lindgen(nx,ny) / nx)

   point = {dipole_point1,B0:0.,P:0.,CMD:0.,LAT:0.,pix_size:[1.,1.],radius:1000.}

   mag = add2str(mag,['bxp','byp','bzp','bx','by','bz','b_pxd','b_pyd','b_long'], $
                  bx,by,bz,bx(*,*,0),by(*,*,0),bz(*,*,0),bpxd,bpyd,bx(*,*,0))
   mag = add2str(mag,['alpha','zscale','point'],alpha,zscale,point)

   return,mag

end
