
;+
;NAME:
;     helio2image
;PURPOSE:
;     computes the rotation matrix for a transformation from heliographic
;     coordinates to image coordinates.
;CATEGORY:
;CALLING SEQUENCE:
;     result = helio2image(b0,p,lat,cmd)
;INPUTS:
;     b0 = Solar b0 angle (radians)
;     p = solar p angle (radians)
;     lat = solar latitude of the center of the FOV (radians)
;     cmd = solar cmd of the center of the FOV (radians)
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;OUTPUTS:
;     result = 3D rotation matrix
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     Does not account for curvature of the sun's surface over the FOV
;
;     DOESN'T WORK!!!!
;PROCEDURE:
;     First the rotation to the correct lat/cmd is done in a single step. 
;     Second, the p-angle rotation is done accounting for the first rotation
;     for the lat/cmd.
;MODIFICATION HISTORY:
;     T. Metcalf  Aug 12, 1992
;-


function helio2image,b0,p,lat,cmd

         ; Find the axis of rotation from the cross-product of the disk
         ; center postion and the observation point.

       ;  rc = [0,0,1] ; Position of disk center
         rc = [0.,sin(b0),cos(b0)] ; Position of disk center
          ; position of obs. point
         ro = [sin(cmd)*cos(lat),sin(lat),cos(cmd)*cos(lat)]

         n = [rc(1)*ro(2)-rc(2)*ro(1), $  ; Cross product (rotation axis)
               rc(2)*ro(0)-rc(0)*ro(2), $
               rc(0)*ro(1)-rc(1)*ro(0)]
         phi = asin(sqrt(total(n^2)))  ; Magnitude of the rotation

         ; Get the rotation matrix for the rotation to obs. point
         junk = v3d_rotate(rc,n,phi,tmatrix=ll_tm)
         if abs(total(junk-ro)) GE 1.0e-4 then begin
            message,'WARNING: Funny business with vectors',/info
            print,'junk:',junk
            print,'ro:',ro
         endif
 
         ; do the b0 rotation about the *old* x-axis

       ;  junk = v3d_rotate(junk,[1,0,0]#invert(ll_tm),b0,tmatrix=b_tm)

       ;  tm = ll_tm # b_tm

         ; Finally do the p-angle rotation by rotating about the *old* z-axis:

         ; Get the tranformation matrix for the p-angle rotation
       ;  junk = v3d_rotate(junk,[0,0,1]#invert(ll_tm),p,tmatrix=p_tm)

       ;  tm = ll_tm # (p_tm # b_tm)
       ;  tm = (ll_tm # b_tm) # p_tm
       ;  tm = (p_tm # b_tm) # ll_tm

         return, ll_tm

end