

function kill_oob, vertices

;+
;NAME:
;     KILL_OOB
;PURPOSE:
;     Deletes lines which extend outside the extrapolation box
;CATEGORY:
;CALLING SEQUENCE:
;     v = kill_oob(v)
;INPUTS:
;     v = Vertices array or structure
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;OUTPUTS:
;     v = Vertices array or structure
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;     Looks for the flag which signals that the line hit the photosphere.
;MODIFICATION HISTORY:
;     T. Metcalf  Oct 6, 1993
;-


   v_is_structure = 0
   vs = size(vertices)
   if vs(vs(0)+1) EQ 8 then begin
      v_is_structure = 1
      if tag_index(vertices,'VERTICES') LT 0 then $
         message,'Vertices structure does not include tag "vertices"'
      v = vertices.vertices
   endif

   if n_elements(v) LE 9 or n_elements(v) MOD 3 NE 0 then $
      message,'Vertices has the wrong format'

   v = reform(v,3,n_elements(v)/3)

   lstart = where(v(0,*) EQ -1000., nlines)  ; -1000 indicates start of line
   lend = where(v(0,*) EQ -2000., nelines)   ; -2000 indicates end of line

   if (nelines NE nlines) then $
      message,/info,'WARNING: # of line ends NE # of line starts'

   if (nlines LE 0) OR (nelines NE nlines) then return,-1

   pointer = 0L
   for line = 0,nlines-1 do begin
      if v(1,lend(line)) EQ 1 then begin      ; if end of line hits photosphere
         aline = v(*,lstart(line):lend(line)) ; then output the line
         naline = n_elements(aline(0,*))
         v(*,pointer:pointer+naline-1)=aline
         pointer = pointer + naline
      endif
   endfor
   v = v(*,0:pointer-1)

   if v_is_structure then begin
      vout = add2str(vertices,delete='VERTICES','VERTICES',v)
      v = temporary(vout)
   endif
   
   return,v
   
end
