;+
;NAME:
;     SUBFF
;PURPOSE:
;     Selects a sub-range of force-free (bff.pro) or vertices (blines.pro) 
;     data.
;CATEGORY:
;CALLING SEQUENCE:
;     sub_structure = subff(structure,range)
;INPUTS:
;     structure = force-free or vertices structure
;     range = [x0,x1,y0,y1] range of pixels to select (original Bgram pixels)
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;OUTPUTS:
;     result = force-free or vertices structure
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     Assumes a very specific format for the input structure
;PROCEDURE:
;MODIFICATION HISTORY:
;     T. Metcalf  December 1992
;-

function subff,mag,range

   smag = size(mag)
   if smag(smag(0)+1) NE 8 then begin   ; Check that the input is a structure
      message,/info,'The input must be a structure'
      return,-1
   endif
   if n_elements(range) NE 4 then begin
      message,/info,'The input range must have 4 elements'
      return,-1
   endif
   range = fix(range)

   bpxd = mag.b_pxd - min(mag.b_pxd)
   bpyd = mag.b_pyd - min(mag.b_pyd)

   hrange = fix( [bpxd(range(0),range(2)), $
                  bpxd(range(1),range(3)), $
                  bpyd(range(0),range(2)), $
                  bpyd(range(1),range(3))] )

   subnxh = hrange(1)-hrange(0)+1
   subnyh = hrange(3)-hrange(2)+1

   ntags = n_tags(mag)
   tnames = strupcase(tag_names(mag))

   for tag = 0,ntags-1 do begin
      case tnames(tag) of 
         'NX': sub=add2str(sub,['NX'],subnxh,/quiet)
         'NY': sub=add2str(sub,['NY'],subnyh,/quiet)
         'VERTICES': begin
                        vpts = where(mag.vertices(0,*) GE 0)
                        if vpts(0) GE 0 then begin
                           v = mag.vertices
                           v(0,vpts) = v(0,vpts) - hrange(0)
                           v(1,vpts) = v(1,vpts) - hrange(2)
                           sub=add2str(sub,['VERTICES'],v,/quiet)
                        endif
                     end
         'BX': sub=add2str(/quiet,sub,['BX'], $
                           mag.bx(hrange(0):hrange(1),hrange(2):hrange(3)))
         'BY': sub=add2str(/quiet,sub,['BY'], $
                           mag.by(hrange(0):hrange(1),hrange(2):hrange(3)))
         'BZ': sub=add2str(/quiet,sub,['BZ'], $
                           mag.bz(hrange(0):hrange(1),hrange(2):hrange(3)))
         'B_PXD': sub=add2str(/quiet,sub,['B_PXD'], $
                           mag.b_pxd(range(0):range(1),range(2):range(3))) 
         'B_PYD': sub=add2str(/quiet,sub,['B_PYD'], $
                           mag.b_pyd(range(0):range(1),range(2):range(3)))
         'BXP': sub=add2str(/quiet,sub,['BXP'], $
                           mag.bxp(hrange(0):hrange(1),hrange(2):hrange(3),*))
         'BYP': sub=add2str(/quiet,sub,['BYP'], $
                           mag.byp(hrange(0):hrange(1),hrange(2):hrange(3),*))
         'BZP': sub=add2str(/quiet,sub,['BZP'], $
                           mag.bzp(hrange(0):hrange(1),hrange(2):hrange(3),*))
         'B_LONG': sub=add2str(/quiet,sub,['B_LONG'], $
                           mag.b_long(range(0):range(1),range(2):range(3))) 
         'B_TRANS': sub=add2str(/quiet,sub,['B_TRANS'], $
                           mag.b_trans(range(0):range(1),range(2):range(3)))
         'B_AZIM': sub=add2str(/quiet,sub,['B_AZIM'], $
                           mag.b_azim(range(0):range(1),range(2):range(3)))
         'I_CONT': sub=add2str(/quiet,sub,['I_CONT'], $
                           mag.i_cont(range(0):range(1),range(2):range(3)))
         'JZ': sub=add2str(/quiet,sub,['JZ'], $
                           mag.jz(hrange(0):hrange(1),hrange(2):hrange(3)))
         'J_TOTAL': sub=add2str(/quiet,sub,['J_TOTAL'], $
                          mag.j_total(hrange(0):hrange(1),hrange(2):hrange(3)))
         else: sub=add2str(/quiet,sub,[tnames(tag)],mag.(tag)) ; Pass others unchanged
      endcase
   endfor

   return,sub

end