;+
; IDL batch file to compile the constant alpha force-free field routines and 
; set up the initial 3D transformation.
;
; To execute this file:  IDL> @bff_compile
;
; A typical use of these routines might be:
;
; IDL> @bff_compile
; IDL> j=bfits('J911115.2105')   ; Read in a magnetic FITS file
; IDL  print,find_alpha(j)       ; Compute a value for alpha
; IDL> m=bff(j,alpha=-1,/surr)   ; Compute the force free field ... OR
; IDL> m=bff(j,/get_alpha,/surr) ; Compute ff field and alpha ... OR
; IDL> v=blines(m,Bcut=800.,skip=2)  ; Plot lines of force ... OR
; IDL> v=blines(m)               ; Plot interactively
; IDL> surfr & scale3d           ; Rotate the 3D image
; IDL> v=blines(v,/vertices)     ; Plot without recomputing lines of force
; IDL> v=blines(v,/vertices,/reselect,magfield=m)  ; Reselect field lines
; IDL> bff_movie,v               ; Make a simple movie
; IDL> fly_blines,v              ; Fly around the 3D plot
;
; Use doc_library to get more information about each of the routines.
;
; You may get an error when compiling some of these routines, due to a lack
; of memory allocated to compilation in IDL.  To fix this execute
;
;   IDL> .size 30000 5000
;
; Caution:  .size will erase all your IDL variables!!
;
;   T. Metcalf   Feb, 1992
;-

.run rfits
.run mkkey_struct
.run rkey
.run strstat
.run get_def
.run makestr
.run tag_index
.run add2str
.run specialct
.run bfits
.run trmvect
.run line_cent
.run runmean
.run get_latlon
.run xform
.run bff/cvt_to_2d
.run bff/subff
.run bicubic
.run bicubic2d
.run bff/bicubic3d
.run bff/binterp
.run bff/rmline
.run bff/bselect
.run binup_b
.run bff/bff
.run bff/blines
.run bff/bline
.run bff/diffeq_tfl
.run bff/bplot
.run bff/bff_movie
.run bff/find_alpha
.run bff/bff_height
.run bff/bhelio
.run bff/fff
.run bff/fff2bff
.run fly
.run bff/fly_blines
.run bff/color_blines
.run bff/kill_oob
.run pixsquare
.run kill_index

t3d,/reset & scale3d
