pro bmagnify,mag,pxd,pyd,b_window=b_window,set_scale=set_scale,length=length,$
             missing=missing,maxT=maxT,levels=levels,LineP_style=LineP_style, $
             LineM_style=LineM_style,scale=iscale,xstep=xstep, $
             no_c_lab=no_c_lab,incolor=incolor,current=current,noblong=noblong,$
             novectors=novectors,image=simage,arrows=arrows, $
             unproject=unproject,tcurrent=tcurrent,print_colors=print_colors, $
             negative=negative,JLevels=jlevels

common ubmagnifystr, unique

;+
;NAME:
;	bmagnify
;PURPOSE:
;	Plots a B map and a zoomed map.
;CATEGORY:
;CALLING SEQUENCE:
;	bmagnify, mag_structure, pxd, pyd, b_window=b_sindow
;INPUTS:
;	mag_structure = a structure containing B info.
;OPTIONAL INPUT PARAMETERS:
;	Many which are passed to Bvector (see comments in Bvector.pro)
;KEYWORD PARAMETERS
;	b_window = specifies the IDL window in which unzoomed B has been plotted
;	set_scale = Set the scale of the zommed image: gives the number of
;	            pixels along half of each dimension (2*set_scale+1) pixels
;	            in each dimension).
;       arrows = If set, put heads on the vectors
;OUTPUTS:
;       A zoomed plot
;COMMON BLOCKS:
;	ubmagnifystr = Used for a static variable
;SIDE EFFECTS:
;	Messes with the color table
;RESTRICTIONS:
;	None
;PROCEDURE:
;MODIFICATION HISTORY:
;	T. Metcalf  Aug 1991
;-

   if n_elements(unique) LE 0 then unique = 0L

   SCALE = 5   ; Number of pixels in zoomed image is 2*SCALE+1
               ; Hence, smaller SCALE means more zoom

   if n_elements(set_scale) GT 0 then SCALE = set_scale

   if n_elements(b_window) EQ 0 then b_window = 0

   if (tag_index(mag,'B_LONG') LT 0)  OR $
      (tag_index(mag,'B_TRANS') LT 0)  OR $
      (tag_index(mag,'B_AZIM') LT 0)  OR $
      (keyword_set(current) AND (tag_index(mag,'JZ') LT 0)) OR $
      (keyword_set(tcurrent) AND (tag_index(mag,'J_TOTAL') LT 0)) OR $
      (keyword_set(unproject) AND (tag_index(mag,'BX') LT 0 OR $
                                   tag_index(mag,'BY') LT 0 OR $
                                   tag_index(mag,'BZ') LT 0 )) then begin
      print,'ERROR: bmagnify: structure does not contain enough information'
      return
   endif 

   nx = n_elements(mag.b_long(*,0))
   ny = n_elements(mag.b_long(0,*))

   xval = findgen(nx)
   yval = findgen(ny)

   wset,b_window 

   window,/free,title='Zoomed Image: Window Active When Filled'
   zoom_window = !D.WINDOW

   wset,b_window
   save_plot = !P
   save_x = !X
   save_y = !Y
  
   print,'Click left button on spot to be magnified'
   print,'Click right button to exit magnifier'

   tvcrs,1            ; enable cursor
   done = 0           ; a flag to end the loop when finished

   REPEAT begin
      cursor,x,y,/down,/device             ; Get place mouse clicked at
      case !D.WINDOW of 
         b_window: begin
                       !P = save_plot    ; Reset the plotting parameters
                       !X = save_x
                       !Y = save_y
                    end
         zoom_window: begin
                        !P = save_zplot    ; Reset the plotting parameters
                        !X = save_zx
                        !Y = save_zy
                      end
      endcase
      spot = convert_coord(x,y,/device,/to_data)   ; Put in data coords
      button_pushed = !ERR

      distance = fltarr(nx*ny)
      for i=0L,nx*ny-1L do distance(i) = (pxd(i MOD nx )-spot(0))^2 + $
                                       (pyd(fix(i/nx))-spot(1))^2
      closest_pixel = sort(distance)
      closest_pixel = closest_pixel(0)
      px = closest_pixel mod nx          ; px,py are the row,col of the pixel
      py = fix(closest_pixel/nx)         ; nearest to the place clicked

      case button_pushed of
         1: begin    ; Left button = magnify
               wset,zoom_window
               xlow = max([0,px-SCALE])
               xhi = min([nx-1,px+SCALE])
               ylow = max([0,py-SCALE])
               yhi = min([ny-1,py+SCALE])
               unique = unique+1

               adv = 0
               REPEAT begin    ; Get a unique structure name
                  check = strstat(strcompress('sub'+string(unique),/remove_all)$
                                  ,/quiet)
                  if check then unique = unique + 1
                  adv = adv + 1
               endrep UNTIL ((NOT check) OR (adv GT 1000))

               if NOT check then begin
                  command = 'sub={sub'+string(unique)
                  if NOT keyword_set(unproject) then begin
                     command = command $
                               + ',b_long:mag.b_long(xlow:xhi,ylow:yhi)' $
                               + ',b_trans:mag.b_trans(xlow:xhi,ylow:yhi)' $
                               + ',b_azim:mag.b_azim(xlow:xhi,ylow:yhi)'
                  endif $
                  else begin
                     command = command  $
                               + ',bx:mag.bx(xlow:xhi,ylow:yhi)'  $
                               + ',by:mag.by(xlow:xhi,ylow:yhi)'  $
                               + ',bz:mag.bz(xlow:xhi,ylow:yhi)'
                  endelse
                  if keyword_set(current) then begin 
                     command = command  $
                               + ',jz:mag.jz(xlow:xhi,ylow:yhi)'  $
                               + ',b_pxd:mag.b_pxd(xlow:xhi,ylow:yhi)' $
                               + ',b_pyd:mag.b_pyd(xlow:xhi,ylow:yhi)' $
                               + ',j_pxd:mag.j_pxd(xlow:xhi,ylow:yhi)' $
                               + ',j_pyd:mag.j_pyd(xlow:xhi,ylow:yhi)' 
                  endif
                  if keyword_set(tcurrent) then begin 
                     command = command  $
                               + ',j_total:mag.j_total(xlow:xhi,ylow:yhi)' 
                  endif
                  command = command + ',point:mag.point'
                  command = command + '}'
                  check = execute(strcompress(command,/remove_all))
               endif $
               else check = 0
               if NOT check then begin
                  print,"ERROR: bmagnify: Can't create structure"
                  return
               endif
               if keyword_set(simage) then dimage = simage(xlow:xhi,ylow:yhi)
               Bvector,sub,incolor=incolor,no_c_lab=no_c_lab,reduce=0, $
                           length=length,missing=missing,maxT=maxT, $
                           levels=levels,LineP_style=LineP_style, $
                           LineM_style=LineM_style,scale=iscale, $
                           ;xstep=xstep, $
                           xval=pxd(xlow:xhi),yval=pyd(ylow:yhi),$
                           current=current,noblong=noblong,novectors=novectors,$
                           image=dimage,/splot,arrows=arrows, $
                           unproject=unproject,tcurrent=tcurrent, $
                           print_colors=print_colors,negative=negative, $
                           Jlevels=jlevels
               save_zplot = !P
               save_zx = !X
               save_zy = !Y
            end
         2:
         4: begin    ; Right button = exit window
               if !D.WINDOW EQ zoom_window then begin
                  erase
                  wset,b_window 
               endif $
               else begin
                  done = 1
                  print,'Exiting ...'
               endelse
            end
         else: print,'Undefined result from cursor',button_pushed
      endcase
   endrep UNTIL done

   wdelete,zoom_window    ; Clean up 

end
