function convert_astruct2nstruct, a, name=name

;+
;NAME:
;     convert_astruct2nstruct
;PURPOSE:
;     convert an anonymous structure to a named structure
;     when substructures are present.
;CATEGORY:
;CALLING SEQUENCE:
;     named_struct = convert_astruct2nstruct(anon_struct)
;INPUTS:
;     anon_struct = anonymous structure
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS:
;     name = name of new structure
;OUTPUTS:
;     named_struct = named structure
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;     Names an anonymous structure and its substructures.  If
;     an anonymous structure has no anonymous substructures, you
;     could just use create_struct.  But this will fail if
;     anonymous substructures are present.  Hence this function.
;MODIFICATION HISTORY:
;     T. Metcalf 2005-Sep-01
;-


   common convert_astruct2nstruct_private,ncall

   ntags = n_tags(a)
   if ntags LE 0 then return,-1

   if n_elements(ncall) LE 0 then ncall = 0L
   ncall = ncall + 1L

   if n_elements(name) LE 0 then begin
      ; if it already has a name, then a is not anonymnous
      ; so just use its name.
      if tag_names(a,/structure_name) then name=tag_names(a,/structure_name) $
      else $
         ; otherwise generate a unique name
         name = strcompress('A'+string(long64(systime(/seconds,/utc)*1000.))+ '_' + $

                            string(ncall),/remove_all)
   endif

   ; Generate the initial structure

   tname = tag_names(a)
   if size(a.(0),/tname) EQ 'STRUCT' then begin
      if tag_names(a.(0),/structure_name) then $
         sname = tag_names(a.(0),/structure_name) $
      else delvarx,sname
      n = create_struct(tname[0],convert_astruct2nstruct(a.(0),name=sname))
   endif else begin
      n = create_struct(tname[0],a.(0))
   endelse

   ; Add the rest of the tags to the structure

   for itag = 1,ntags-1 do begin
      if size(a.(itag),/tname) EQ 'STRUCT' then begin
         if tag_names(a.(itag),/structure_name) then $
            sname = tag_names(a.(itag),/structure_name) $
         else delvarx,sname
         n = create_struct(n,tname[itag],convert_astruct2nstruct(a.(itag),name=sname))
      endif else begin
         n = create_struct(n,tname[itag],a.(itag))
      endelse
   endfor

   ; give the structure its name

   return,create_struct(n,name=name)

end
