;*******************************************************************************
;
; Procedure ACentsolcor
; Calculating the heliocentric coordinates of the center point of a scan.
;
; Input:
; 1) B0: heliocentric lattitude of the solar disk center
; 2) P:  the angle between the solar spin axis and the north of solardisk
;        projected onto the solar disk.
; 3) Solarradius: the radius of the sun in arcsec
; 4) Centerfast, Centerslow: Center raster point IN ARCSEC FROM DISK CENTER
; 
; Output:
; 1) LC: the heliocentric longitude of the center point of the scan
; 2) BC: the heliocentric lattitude of the center point of the scan
;    The output are all in radiants.
;
;*******************************************************************************
PRO acentsolcor, b0,p,solarradius,centerfast,centerslow,lc,bc
rs=solarradius
xc=centerfast
yc=centerslow
arc=!pi/180./3600.
rho=sqrt(xc^2+yc^2)
rhorad=arc*rho
seta=acos(yc/rho)
if xc lt 0. then seta=-seta
rss=rs*arc
sunrho=asin(rhorad/rss)-rhorad
bc=asin(sin(b0)*cos(sunrho)+cos(b0)*sin(sunrho)*cos(p-seta))
lc=asin(sin(sunrho)*sin(p-seta)/cos(bc))
end
