
PRO Jzhigh, bdxarraym,bdyarraym,bdzarraym,pxd,pyd,b0,p,lc,bc,$
   jzarrayh,jpxd,jpyd,pix_size,noplot=noplot,jz_window=jz_window, $ 
   open_window=open_window,noask=noask,scale=scale,nx_big=nx_big,ny_big=ny_big,$
   xoffset=xoffset,yoffset=yoffset,noerase=noerase,title=title

;+
;NAME:
;	JZHIGH
;PURPOSE:
;	Calculate and plot the vertical current of the magnetic field.
;	The calculation is done by using the image coordinates instead of doing
;	interpolation.  It is much faster than doing interpolation and keep the
;	resolution of the original measured data.
;CATEGORY:
;CALLING SEQUENCE:
;     Jzhigh,bdxarraym,bdyarraym,bdzarraym,pxd,pyd,b0,p,lc,bc,jzarrayh,jpxd,jpyd
;           ,pixsize
;INPUTS:
;	Bdxarraym: X component of the photospherice magnetic field
;	Bdyarraym: Y component of the photospheric magnetic field
;	Bdzarraym: Normal component of the photospheric magnetic field
;       pxd: X coordinate on the photosphere for each of the pixel
;		in the above array 
;	pyd: Y coordinate on the photosphere for each of the pixel
;		in the above array
;	B0: The heliocentric lattitude of disk center
;	P: The angle between the disk projection of the solar spin axis
;		and the north point of the disk. (eastward is positive)
;		(both B0 and P are in radians)
;	LC: The heliocentric longitude of the center point of the scan
;		(in radians)
;	BC: The heliocentric lattitude of the center point of the scan
;		(in radians)
;	pix_size: the size of the pixels in arc seconds (5.656 default)
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;	noplot = If set, no plot is generated
;	jz_window = Specifies the windoe number to plot in
;	open_window = If set, open a window with the 'window' command
;	noask = If set, do not ask whethere alpha histogram should be computed
;	scale = Specifies the scale of the plot: 1.0 is default, 1.2 is 20%
;		larger etc.  Scale is only used when open_window is set.
;       noerase = If set, do not erase the window before plotting
;       title = title for plot
;
;	The following are used to correctly compute/plot a subarray of a larger
;	field raster:
;	nx_big = The number of x-pixels in the large raster
;	ny_big = The number of y-pixels in the large raster
;	xoffset = The x-offset (in pixels) from the center of the large array to
;		  the center of the sub-array.
;	yoffset = The y-offset (in pixels) from the center of the large array to
;		  the center of the sub-array.
;	    Following is an example of a call to jzhigh with a subarray:
;                       jzhigh,bdx(xlow:xhi,ylow:yhi), $
;                              bdy(xlow:xhi,ylow:yhi), $
;                              bdz(xlow:xhi,ylow:yhi), $
;                              pxd(xlow:xhi,ylow:yhi), $
;                              pyd(xlow:xhi,ylow:yhi), $
;                              b0,p,pos_cmd,pos_lat,tjz,tjpxd,tjpyd, $
;                              jz_window=window_value,/noask, $
;                              nx_big = nx, $ ;nx is the x-size of the big array
;                              ny_big = ny, $ ;ny is the y-size of the big array
;                              xoffset = xlow + (xhi-xlow+1)/2. - nx/2., $
;                              yoffset = ylow + (yhi-ylow+1)/2. - ny/2.
;OUTPUTS:
;	jzarrayh:Vertical current
;	jpxd: The X coordinate on the photosphere for each pixel in
;		jzarrayh array
;	jpyd: The Y coordinate on the photosphere for each pixel in
;		jzarrayh array
;	Photospheric vector magnetogram superimposed by vertical current
;COMMON BLOCKS:
;	none
;SIDE EFFECTS:
;	Messes with the color table
;RESTRICTIONS:
;	none
;PROCEDURE:
;MODIFICATION HISTORY:
;	Original by Y. Fan
;	Modified by TRM      5/30/91
;-

ss=size(bdxarraym)
nx=ss(1)
ny=ss(2)
cxx=cos(p)*cos(lc)-sin(p)*sin(b0)*sin(lc)
cxy=(-1)*cos(p)*sin(bc)*sin(lc)-sin(p)*(cos(b0)*cos(bc)+sin(b0)* $
sin(bc)*cos(lc))
cyx=sin(p)*cos(lc)+cos(p)*sin(b0)*sin(lc)
cyy=(-1)*sin(p)*sin(bc)*sin(lc)+cos(p)*(cos(b0)*cos(bc)+ $
sin(b0)*sin(bc)*cos(lc))

if n_elements(nx_big) EQ 0 then nx_big = nx
if n_elements(ny_big) EQ 0 then ny_big = ny
if n_elements(xoffset) EQ 0 then xoffset = 0
if n_elements(yoffset) EQ 0 then yoffset = 0
if n_elements(pix_size) EQ 0 then begin
   pix_size = [5.656,5.656]
   print,'JZHIGH: Using default pixel size of 5.656 arcseconds'
endif

x0=(nx/2.-xoffset)*(pix_size(0)/min(pix_size))
y0=(ny/2.-yoffset)*(pix_size(1)/min(pix_size))
delt=cxx*cyy-cxy*cyx
   lylxbx=shift(bdxarraym,-1,-1)
   lylxby=shift(bdyarraym,-1,-1)
   lxbx=shift(bdxarraym,-1,0)
   lxby=shift(bdyarraym,-1,0)
   lybx=shift(bdxarraym,0,-1)
   lyby=shift(bdyarraym,0,-1)
   bx=bdxarraym
   by=bdyarraym
   dev1=0.5*(lylxby+lxby-lyby-by)  ; dBy/dx   cxx = dx/dX
   dev2=0.5*(lylxby+lyby-lxby-by)  ; dBy/dy   cyx = dy/dX
   dev3=0.5*(lylxbx+lxbx-lybx-bx)  ; dBx/dx   cxy = dx/dY
   dev4=0.5*(lylxbx+lybx-lxbx-bx)  ; dBx/dy   cyy = dy/dY
; The constant 0.0193 is derived as follows:
; 10^3 * 10^(-4) / (mu * pxlength)
; where mu = 4*pi*10^(-7)
; pxlength = length of each pixel in meters (5.7 arcsec corresponds to 4.13e6 m)
; 10^3 is the conversion between mAmp and Amp
; 10^(-4) is the conversion between Gauss and Tesla
; jz is thus in units of mAmp/m^2
   jzarrayh=0.0193*(cxx*dev1+cyx*dev2-cxy*dev3-cyy*dev4)*(5.656/min(pix_size))
   jzarrayh(nx-1,*)=0.
   jzarrayh(*,ny-1)=0.

   if n_elements(jpxd) NE n_elements(pxd) then begin
      xcoor=(indgen(nx)#replicate(1,ny)+0.5)*pix_size(0)/min(pix_size)
      ycoor=(replicate(1,nx)#indgen(ny)+0.5)*pix_size(1)/min(pix_size)
      dircos=cos(bc)*cos(lc)*cos(b0)+sin(bc)*sin(b0)
      nmax=fix(sqrt(float(nx_big*nx_big+ny_big*ny_big))/dircos/2.)
      jpxd=(cyy*(xcoor-x0)-cxy*(ycoor-y0))/delt+nmax
      jpyd=(cxx*(ycoor-y0)-cyx*(xcoor-x0))/delt+nmax
      jzarrayh(0,*)=0.
      jzarrayh(*,0)=0.
      jpxd(0,*)=pxd(0,*)
      jpxd(*,0)=pxd(*,0)
      jpxd(nx-1,*)=pxd(nx-1,*)
      jpxd(*,ny-1)=pxd(*,ny-1)
      jpyd(0,*)=pyd(0,*)
      jpyd(*,0)=pyd(*,0)
      jpyd(nx-1,*)=pyd(nx-1,*)
      jpyd(*,ny-1)=pyd(*,ny-1)
   endif

   xsize=max(pxd)-min(pxd)
   ysize=max(pyd)-min(pyd)

   if (NOT keyword_set(noplot)) then begin

       bhelio_current,bdxarraym,bdyarraym,bdzarraym,pxd,pyd,jzarrayh,jpxd,jpyd, $
          b_window=jz_window, open_window=open_window,scale=scale,xstep=xstep, $
          ystep=ystep,current=current,length=length,noerase=noerase,title=title

      if (NOT keyword_set(noask)) then begin
         print,'Do you want to draw alpha histogram? Yes: y , No: n'
         answer='ansewer'
         read,answer
         if answer eq 'n' then goto,finish
         print,'You want to choose pixels with the magnitude of vertical current and'
         print,'normal magnetic field strength greater than j0 and Bz0 respectively'
         print,'Tell me the value of j0 and Bz0 please. '
         print,'j0='
         read,j0
         print,'Bz0='
         read,bz0
         ;smoothing the vertical magnetic field
            bdzarr=0.25*(shift(bdzarraym,-1,-1)+shift(bdzarraym,-1,0)+ $
                   shift(bdzarraym,0,-1)+bdzarraym)
         bdzarr(*,ny-1)=0.
         bdzarr(nx-1,*)=0.
         ncrit=where(abs(bdzarr) lt bz0 or abs(jzarrayh) lt j0)
         crit=where(abs(bdzarr) ge bz0 and abs(jzarrayh) ge j0)
         ;nonzero the bdzarr array
         bdzarr(ncrit)=fix(bdzarr(ncrit))+0.1
         alpha=jzarrayh/bdzarr
         alpha(ncrit)=0.
         print,'Do you want to take a look at the position of those pixels chosen?'
         print,'Yes: y , No: n'
         read,answer
         if answer eq 'n' then goto, skip
         print,'Tell me the window number you want to plot'
         read,nw
         window,nw,xs=xsize*10,ys=ysize*10
         whitebg
            contour,bdzarr,jpxd,jpyd,levels=lv,c_color=(lv lt 0)*100+100,$
            xstyle=1,ystyle=1
            oplot,jpxd(crit),jpyd(crit),color=1,psym=1
         skip: critplus=where(bdzarr ge bz0 and abs(jzarrayh) ge j0)
               critminus=where(bdzarr le -(bz0) and abs(jzarrayh) ge j0)
            alfaplus=alpha(critplus)
            alfaminus=alpha(critminus)
            sss=[min(alfaminus),min(alfaplus)]
            ttt=[max(alfaminus),max(alfaplus)]
            dimlow=min(sss)
            dimhigh=max(ttt)
            bdzarrplus=bdzarr(critplus)
            bdzarrminus=bdzarr(critminus)
            alphaerr=4./bz0
            dim=fix((dimhigh-dimlow)/alphaerr)+1
            fluxplus=fltarr(dim)
            fluxminus=fltarr(dim)
            scr=size(bdzarrplus)
            imaxplus=scr(1)
            for i=0, imaxplus-1 do begin
            binalpha=fix((alfaplus(i)-dimlow)/alphaerr)
            fluxplus(binalpha)=fluxplus(binalpha)+bdzarrplus(i)
            endfor
            scr = size(bdzarrminus)
            imaxminus=scr(1)
            for i= 0, imaxminus-1 do begin
            binalpha=fix((alfaminus(i)-dimlow)/alphaerr)
            fluxminus(binalpha)=fluxminus(binalpha)+bdzarrminus(i)
            endfor
         print,'**fluxplus**'
         print,fluxplus
         print,'**fluxminus**'
         print,fluxminus
         print,'Tell me the number for the window in which you want to plot the '
         print,'alpha histogram'
         read,nw
         window,nw
         pxlength=5.7*(1.496e+11)*!pi/3600./180.
         pxarea=pxlength^2/delt
         fluxplus=fluxplus*pxarea*(1.e+4)/(1.e+20)
         fluxminus=fluxminus*pxarea*(1.e+4)/(1.e+20)
         xline=(indgen(dim)*alphaerr+dimlow+0.5*alphaerr)
         plot,xline,-fluxminus,psym=-7,xrange=[-0.1,0.1],yrange=[0,50.],$
         xtitle='alpha (Gauss*sqM/mA)',ytitle='Bflux (10^20 Maxwell)'
         xline=(indgen(dim)*alphaerr+dimlow+0.5*alphaerr)
         oplot,xline,fluxplus,psym=-6,$
         xtitle='alpha (Gauss*sqM/mA)',ytitle='Bflux (10^20Maxwell)'
         jzbala=total(jzarrayh(crit))*pxarea*(1.e-3)
         jzabs=total(abs(jzarrayh(crit)))*pxarea*(1.e-3)
         bzbala=total(bdzarr(crit))*pxarea*(1.e+4)
         bzabs=total(abs(bdzarr(crit)))*pxarea*(1.e+4)
         jzper=abs(jzbala)/jzabs
         bzper=abs(bzbala)/bzabs
         jzsumerr=sqrt(n_elements(crit))*j0*pxarea*(1.e-3)
         bzsumerr=sqrt(n_elements(crit))*bz0*pxarea*(1.e+4)
         print,'**************************Table 1 *******************************'
         print,' total jz	total |jz|	total bz	total |bz| '
         print,' (A)		(A)		(Maxwell)	(Maxwell)'
         print, jzbala,jzabs,bzbala,bzabs
         print,'total jz err	total bz err 	pixel number'
         print,jzsumerr,bzsumerr,n_elements(crit)
         print,'******************************************************************'
      endif
   endif
   finish: ;print,'finished'
   end
