PRO VFIELD2,U,V,X,Y,thick=th,color=col,background=back,Length=length,title=title
;
;NAME:
;      VFIELD2
;PURPOSE:
;      Produce a two dimentional vector field plot.
;              Draw a directed arrow at each point showing
;              the direction and matnitude of the field.
;
;CATEGORY:
;      Plotting
;CALLING SEQUENCE:
;      VEFIELD2,U,V,X,Y
;INPUTS:
;      U = The X component of the two dimensional field.  Both
;              U and V must be two dimensional and have the same
;              dimensions
;      V = The Y component of the two dimensional field.
;      X = The X coordinants of the field points.  It should be two
;              dimensional and have the same size as U and V.
;      Y = The y coordinates of the field points.  It should be two
;              dimensional and have the same size as U and V.
;Keyword Input Parameters:
;      Length = Length factor, default=1.0 makes the longest (U,V) the
;               length of diagonal of a cell.
;      thick = thickness of the line.  Must be specified.
;      title = title for the plot
;OUTPUTS:
;      None.
       s=size(u)
       xsz=(max(x)-min(x))/s(1)
       ysz=(max(y)-min(y))/s(2)
       unitlen=sqrt(xsz^2+ysz^2)
       maxmag=max(sqrt(u^2+v^2))   
       mag=sqrt(u^2+v^2)
       len=length*mag/maxmag*unitlen
;
       plot,[min(x),max(x)],[max(y),min(y)],background=back,color=col,$
           /nodata,/xst,/yst,/noer,title=title
;
       for j=0,s(2)-1 do begin
        for i=0,s(1)-1 do begin
          x0=x(i,j)
          y0=y(i,j)
          if (mag(i,j) ne 0.) then begin
            theta=acos(u(i,j)/mag(i,j))
            if (v(i,j) lt 0.) then theta=2*!pi-theta
            dx=len(i,j)*cos(theta)
            dy=len(i,j)*sin(theta)
            arrowlen=0.3*len(i,j)
            dx1=-arrowlen*cos(theta-30.*!pi/180.)
            dy1=-arrowlen*sin(theta-30.*!pi/180.)
            dx2=-arrowlen*cos(theta+30.*!pi/180.)
            dy2=-arrowlen*sin(theta+30.*!pi/180.)
            endif else begin
            dx=0. & dy=0. & dx1=0. & dy1=0. & dx2=0. & dy2=0.
          endelse
          x1=x0+dx & y1=y0+dy
          x2=x1+dx1 & y2=y1+dy1
          x3=x1+dx2 & y3=y1+dy2 
          plots,[x0,x1,x2,x1,x3],[y0,y1,y2,y1,y3],thick=th,color=col
         endfor
        endfor
end 
