
function bterror,j,verbose=verbose,binsize=binsize,maxvalue=maxv

;+
;NAME:
;     BTERROR
;PURPOSE:
;     Computes the one sigma error in the transverse magnetic field
;CATEGORY:
;CALLING SEQUENCE:
;     bterr = bterror(mag)
;INPUTS:
;     mag = magnetic field structure containing the B_TRANS tag,
;           OR an array of transverse field values.
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;     /verbose = Print the value of the error
;OUTPUTS:
;     bterr = the one sigma error in the transverse magnetic field.
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;     Looks for the peak in the b_trans histogram.
;MODIFICATION HISTORY:
;      T. Metcalf 07-Apr-94
;      TRM 2004-Apr-23 Fit a Gaussian to the histogram before finding
;                      the maximum.  Default maxv is 699. changed from 1000.
;                      Default binsize is 10 changed from 20.  Now
;                      this routine gets the same answer as
;                      get_berrs.pro. 
;-

   ; Compute error in the transverse magnetic field
 
   s=size(j)
   if s(s(0)+1) EQ 8 then bt=j.b_trans else bt=j
   if n_elements(binsize) NE 1 then binsize = 10.0
   if n_elements(maxv) NE 1 then maxv = 699.
   ;hs = rotate(sort(histogram(abs(bt),bin=binsize,min=0.,max=maxv,locations=xx)),2)
   ;bsigma = (hplace+0.5)*binsize  ; 1 sigma error in the magnetic field
   yy = histogram(abs(bt),min=1,max=maxv,binsize=binsize,locations=xx)
   xx = xx + binsize/2.0
   aa=[max(yy),(xx(where(yy eq max(yy))))(0),100,0.,0.,0.]
   fit = gaussfit(xx,yy,aa,nterms=6)
   hs = rotate(sort(fit),2)
   if hs(0) NE 0 then hplace = hs(0) else hplace=hs(1)
   ; 1 sigma error in the transverse magnetic field at histogram peak
   bsigma = xx[hplace]  

   if keyword_set(verbose) then $
      message,/info,strcompress('B sigma = '+string(long(bsigma)))+' G'
 
   return,bsigma

end
