
;+

function fffresolve,j,alpha,guard=guard,shear=shear

;NAME:
;     FFFRESOLVE
;PURPOSE:
;     Resolve the 180 degree ambiguity using a linear force free
;     field for comparison
;CATEGORY:
;CALLING SEQUENCE:
;     j = fffresolve(b,alpha)
;INPUTS:
;     b = magnetic field structure to be resolved
;     alpha = linear FFF parameter (0.0 = potential)
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;     guard = guard width for FFF.
;     shear = returns with the shear angle between the FFF and the
;             gram azimuth.
;OUTPUTS:
;     j = resolved magnetic field structure.
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     Azimuth is measured CCW from north
;PROCEDURE:
;MODIFICATION HISTORY:
;     T. Metcalf 25-Apr-2003
;     2005-Sep-08 TRM Added apodize keyword (passed to FFF).
;-


   p = double(j.point.p)
   b0 = double(j.point.b0)
   if tag_index(j,'CMD',/quiet) GE 0 then lc = double(j.cmd)*!dtor $
   else lc = double(j.point.cmd)
   if tag_index(j,'LATITUDE',/quiet) GE 0 then bc = double(j.latitude)*!dtor $
   else bc = double(j.point.lat)
   pix_size = double(j.point.pix_size)
   if n_elements(pix_size) EQ 1 then pix_size = [pix_size[0],pix_size[0]]

   pot = fff(b0=b0,pang=p,cmd=lc,lat=bc,j.b_long,/image,pixel=pix_size, $
             alpha=alpha,/quiet,guard=guard)
   pxi = pot(*,*,0)  ; These are image coordinates
   pyi = pot(*,*,1)
   pzi = pot(*,*,2)

   check = check_math(0,1)
      pa_azimuth = atan(-pxi,pyi)/!dtor  ; Az is CCW from North
   check = check_math(1,0)

   return,fix_azimuth(j,pa_azimuth,shear=shear)

end
