function iscale,data,decimal,scale

;+
;NAME:
;     ISCALE
;PURPOSE:
; Scales data by a power of 10 to minimize loss of accuracy when converting
; to interger values, and then convers to integer.
;CATEGORY:
;CALLING SEQUENCE:
;     result = iscale(data,decimal,scale)
;INPUTS:
;     data = data to be converted to integer type
;     decimal = the ideal number of decimal places to preserve in the data.
;               if the values are too large, fewer decimal places may be 
;               preserved.
;OPTIONAL INPUT PARAMETERS:
;     none
;KEYWORD PARAMETERS
;     none
;OUTPUTS:
;     result = data converted to integer
;     scale = the value by which the data is scaled.  Multiply the integer
;             data by scale to recover the original data (thouh some accuracy
;             may be lost).
;COMMON BLOCKS:
;     none
;SIDE EFFECTS:
;     none
;RESTRICTIONS:
;     When the data is converted to integer, some accuracy may be lost.
;     The maximum number of decimal places is 4.
;PROCEDURE:
;     Scales the data by a power of 10 to minimize data loss when converting
;     to integer.
;MODIFICATION HISTORY:
;     TRM   Oct, 1991
;-

  if n_elements(decimal) EQ 0 then decimal=1
  if (decimal GT 4) then decimal=4

  scale = 10.^(decimal+1)

  REPEAT begin
     scale = scale/10.
     test = data*scale
  endrep UNTIL max(abs(test)) LT 32768

  scale = 1.0/scale

  return,fix(test)

end
