
;+
; NAME:
;        pp_reverse
; PURPOSE:
;        Calculate the "pp" vector and the "transtype" for the inverse
;        transform
; CATEGORY:
; CALLING SEQUENCE:
;        pp_reverse,pp_in,pp_out
;        pp_reverse,pp_in,pp_out,ttyp_in,ttyp_out
; INPUTS:
;        pp_in = fltarr(2,2,*).  Point array defining transform.
;        ttyp_in = string, float element, or structure.  Sets constraints.
;                  See "transtype" keyword in CALTRANS for details.
; KEYWORDS (INPUT):
; OUTPUTS:
;        pp_out = fltarr(2,2,*).  Point array defining inverse transform.
;        ttyp_out = string or structure.  Constraints for inverse transf.
; KEYWORDS (OUTPUT):
; COMMON BLOCKS:
;        None.
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;        JPW, 22-nov-94
;-

pro pp_reverse,ppi,ppo,tti,tto

ppo = ppi
ppo(*,0,*) = ppi(*,1,*)
ppo(*,1,*) = ppi(*,0,*)

tysi = size(tti)
case tysi(tysi(0)+1) of
 0 : tto = 'g'
 7 : tto = tti
 8 : begin
      tto = tti
      tto.pix_x = tti.ref_x
      tto.pix_y = tti.ref_y
      tto.ref_x = tti.pix_x
      tto.ref_y = tti.pix_y
      if tti.transpose eq 0 then tto.phi = -tti.phi
     end
 else : begin
      tto = {transtype_struct,type:'i',pix_x:1.0,pix_y:1.0, $
             ref_x:1.0,ref_y:1.0,transpose:0,phi:0.0}
      tto.ref_y = abs(tti)
      if tti lt 0.0 then tto.transpose = 1
     end
endcase

return
end

