;+
;NAME:
;	remapj
;PURPOSE:
;       Remaps Jz onto the field grid
;CATEGORY:
;CALLING SEQUENCE:
;       jnew = remapjz(j)
;INPUTS:
;       j = magnetic field structure
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;       /total = return the total current rather than the vertical current
;OUTPUTS:
;       jnew = the remapped jz
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;       if the input is not a structure, the input is passed to the output
;PROCEDURE:
;MODIFICATION HISTORY:
;       T. Metcalf 1992
;       T. Metcalf 1996-10-02 Change poly_2d interpolation to 2 from 1.
;       T. Metcalf 2005-07-06 Added cubic=-0.5 keyword to poly_2d call.
;-

function remapjz,j,total=total

     sj = size(j)
     if sj(sj(0)+1) NE 8 then begin
          message,/info,'J is not a structure ... no mapping'
          return,j  ; If not structure, do nothing
     endif
     degree = (sqrt(n_elements(j.b_pxd))-1.0) < 1
     polywarp,j.b_pxd,j.b_pyd,j.j_pxd,j.j_pyd,long(degree),kx,ky
     if keyword_set(total) then $
        return,poly_2d(j.j_total,kx,ky,2,cubic=-0.5) $
     else $
        return,poly_2d(j.jz,kx,ky,2,cubic=-0.5)

end
