
function rkey,header,keyword,ktype,num

;+
; NAME:
;        rkey
; PURPOSE:
;        Extracts a FITS value from a FITS header.
; CATEGORY:
;        FITS
; CALLING SEQUENCE:
;        result = rkey(header,keyword,ktype,num)
; INPUTS:
;        header = string array containing lines of FITS header
;	 keyword = string containing the keyword to extract
;        ktype   = string containing type of keyword 
;		   (INTEGER,REAL,CHAR,LOGICAL,COMMENT)
;        num = Index for multiple keywords like HISTORY and COMMENT, eg if
;              num is 2, take the second occurance of the keyword.
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;	 None
; OUTPUTS:
;        result = integer,float, or string  depending on keyword type.
;                 If the keyword is not found, result is returned equal 
;                 to 'undefined'
; OPTIONAL (KEYWORD) OUTPUT PARAMETERS:
;        None
; COMMON BLOCKS:
;        None.
; SIDE EFFECTS:
;        None.
; RESTRICTIONS:
;        None.
; MODIFICATION HISTORY:
;        TRM, Nov, 1990
;-


nhead = n_elements(header)

value='undefined'    ; default value to return

if n_elements(num) EQ 0 then num = 1

count = 0

for i=0,nhead-1 do begin
   if (strcompress(keyword,/remove_all) EQ $
       strcompress(strmid(header(i),0,8),/remove_all) ) then begin
     count = count + 1
     CASE strcompress(ktype,/remove_all) OF
     'INTEGER':  $
        value = fix(strcompress(strmid(header(i),10,20),/remove_all))
     'REAL':  $
        value = float(strcompress(strmid(header(i),10,20),/remove_all))
     'CHAR':  begin
        value = strtrim(strmid(header(i),10,20),2)
        value = strmid(value,1,strlen(value)-2)
      end
     'LOGICAL': begin
        value = strmid(header(i),29,1)
        CASE value OF
        'T': value = 1
        'F': value = 0
        ELSE: begin
           print,'WARNING: rkey: Invalid LOGICAL encountered',value
           value = 0
         end
        ENDCASE
      end
     'COMMENT':  $
        value = strtrim(strmid(header(i),10,70))
     ELSE: begin
         print,'WARNING: rkey: Invalid keyword type encountered',ktype
         value = 0
      end
     endcase
     if num EQ count then return,value
   endif
endfor

if (string(value) EQ 'undefined') then $
  print,'WARNING: rkey: keyword ',keyword,' not found'

return,value

end

