;+
;NAME:
;     runmean
;PURPOSE:
;     Running mean of a vector or matrix
;CATEGORY:
;CALLING SEQUENCE:
;     m = runmean(x,[width,ywidth])
;INPUTS:
;     x = vector (1D) or matrix (2D)
;     width = Integer width of running mean (default = 3)
;     ywidth = Integer width of running mean in the y direction for a matrix 
;              Default = same as width
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;     pixels = a 1D vector of pixels at which the running mean is to be 
;              computed.  All other pixels are left with the same value they
;              had on input.
;OUTPUTS:
;     m = running mean, vector with same dimension as x
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;MODIFICATION HISTORY:
;     TRM  April 13, 1993
;     TRM  August 3, 1993  Added 2D matrix computations
;-

function runmean,x,width,ywidth,pixels=pixels

  sx = size(x)
  if n_elements(width) NE 1 then width = 3L
  wid = long(width)
  if n_elements(ywidth) NE 1 then ywidth = wid
  ywid = long(ywidth)
  if wid MOD 2 EQ 0 then message,/info,'It is better to use an odd width' $
  else if ywid MOD 2 EQ 0 then message,/info,'It is better to use an odd width'
  if wid EQ 1 and ywid EQ 1 then return,x

  if n_elements(pixels) LE 0 then pixels = lindgen(n_elements(x)) $
  else pixels=long(pixels)
  npix = n_elements(pixels)

  a = x
  w = lindgen(wid)-wid/2L
  nx = sx(1)

  if sx(0) EQ 1 then begin
     for i=0L,npix-1L do a(pixels(i))=total(x((((w+i)>0)<(nx-1))))/wid
  endif $
  else begin
     ny = sx(2)
     wy = lindgen(ywid)-ywid/2L
     w1 = wid-1
     yw1 = ywid-1
     xywid = wid*ywid
     for pix=0L,npix-1L do begin
        i = pixels(pix) MOD nx
        j = pixels(pix) / nx
        iw = ((i+w)>0)<(nx-1)
        jw = ((j+wy)>0)<(ny-1)
        a(i,j) = total(x(iw(0):iw(w1),jw(0):jw(w1)))/xywid
     endfor
  endelse
  return,a

end
